@@ -71,19 +71,39 @@ BOTOCORE_DEFAUT_SESSION_VARIABLES = {
     'metadata_service_num_attempts': (
         'metadata_service_num_attempts',
         'AWS_METADATA_SERVICE_NUM_ATTEMPTS', 1, int),
+    'ec2_metadata_service_endpoint': (
+        'ec2_metadata_service_endpoint',
+        'AWS_EC2_METADATA_SERVICE_ENDPOINT',
+        None, None),
+    'ec2_metadata_service_endpoint_mode': (
+        'ec2_metadata_service_endpoint_mode',
+        'AWS_EC2_METADATA_SERVICE_ENDPOINT_MODE',
+        None, None),
+    'imds_use_ipv6': (
+        'imds_use_ipv6',
+        'AWS_IMDS_USE_IPV6',
+        False, utils.ensure_boolean),
+    'use_dualstack_endpoint': (
+        'use_dualstack_endpoint',
+        'AWS_USE_DUALSTACK_ENDPOINT',
+        None, utils.ensure_boolean),
+    'use_fips_endpoint': (
+        'use_fips_endpoint',
+        'AWS_USE_FIPS_ENDPOINT',
+        None, utils.ensure_boolean),
     'parameter_validation': ('parameter_validation', None, True, None),
     # Client side monitoring configurations.
     # Note: These configurations are considered internal to botocore.
     # Do not use them until publicly documented.
     'csm_enabled': (
-            'csm_enabled', 'AWS_CSM_ENABLED', False, utils.ensure_boolean),
+        'csm_enabled', 'AWS_CSM_ENABLED', False, utils.ensure_boolean),
     'csm_host': ('csm_host', 'AWS_CSM_HOST', '127.0.0.1', None),
     'csm_port': ('csm_port', 'AWS_CSM_PORT', 31000, int),
     'csm_client_id': ('csm_client_id', 'AWS_CSM_CLIENT_ID', '', None),
     # Endpoint discovery configuration
     'endpoint_discovery_enabled': (
         'endpoint_discovery_enabled', 'AWS_ENDPOINT_DISCOVERY_ENABLED',
-        False, utils.ensure_boolean),
+        'auto', None),
     'sts_regional_endpoints': (
         'sts_regional_endpoints', 'AWS_STS_REGIONAL_ENDPOINTS', 'legacy',
         None
@@ -117,7 +137,20 @@ DEFAULT_S3_CONFIG_VARS = {
         ['s3_us_east_1_regional_endpoint',
          ('s3', 'us_east_1_regional_endpoint')],
         'AWS_S3_US_EAST_1_REGIONAL_ENDPOINT', None, None
-    )
+    ),
+    's3_disable_multiregion_access_points': (
+        ('s3', 's3_disable_multiregion_access_points'),
+        'AWS_S3_DISABLE_MULTIREGION_ACCESS_POINTS', None, utils.ensure_boolean
+    ),
+}
+# A mapping for the proxy specific configuration vars. These are
+# used to configure how botocore interacts with proxy setups while
+# sending requests.
+DEFAULT_PROXIES_CONFIG_VARS = {
+    'proxy_ca_bundle': ('proxy_ca_bundle', None, None, None),
+    'proxy_client_cert': ('proxy_client_cert', None, None, None),
+    'proxy_use_forwarding_for_https': (
+        'proxy_use_forwarding_for_https', None, None, utils.normalize_boolean),
 }
 
 
@@ -129,6 +162,10 @@ def create_botocore_default_config_mapping(session):
         's3', session, _create_config_chain_mapping(
             chain_builder, DEFAULT_S3_CONFIG_VARS)
     )
+    config_mapping['proxies_config'] = SectionConfigProvider(
+        'proxies_config', session, _create_config_chain_mapping(
+            chain_builder, DEFAULT_PROXIES_CONFIG_VARS)
+    )
     return config_mapping
 
 
