@@ -65,6 +65,17 @@ class UnknownServiceError(DataNotFoundError):
         "{known_service_names}")
 
 
+class UnknownRegionError(BotoCoreError):
+    """Raised when trying to load data for an unknown region.
+
+    :ivar region_name: The name of the unknown region.
+
+    """
+    fmt = (
+        "Unknown region: '{region_name}'. {error_msg}"
+    )
+
+
 class ApiVersionNotFoundError(BotoCoreError):
     """
     The data associated with either the API version or a compatible one
@@ -78,6 +89,7 @@ class ApiVersionNotFoundError(BotoCoreError):
 
 class HTTPClientError(BotoCoreError):
     fmt = 'An HTTP Client raised an unhandled exception: {error}'
+
     def __init__(self, request=None, response=None, **kwargs):
         self.request = request
         self.response = response
@@ -96,6 +108,13 @@ class InvalidIMDSEndpointError(BotoCoreError):
     fmt = 'Invalid endpoint EC2 Instance Metadata endpoint: {endpoint}'
 
 
+class InvalidIMDSEndpointModeError(BotoCoreError):
+    fmt = (
+        'Invalid EC2 Instance Metadata endpoint mode: {mode}'
+        ' Valid endpoint modes (case-insensitive): {valid_modes}.'
+    )
+
+
 class EndpointConnectionError(ConnectionError):
     fmt = 'Could not connect to the endpoint URL: "{endpoint_url}"'
 
@@ -123,6 +142,10 @@ class ProxyConnectionError(ConnectionError, requests.exceptions.ProxyError):
     fmt = 'Failed to connect to proxy URL: "{proxy_url}"'
 
 
+class ResponseStreamingError(HTTPClientError):
+    fmt = 'An error occurred while reading from response stream: {error}'
+
+
 class NoCredentialsError(BotoCoreError):
     """
     No credentials could be found.
@@ -186,6 +209,19 @@ class NoRegionError(BaseEndpointResolverError):
     fmt = 'You must specify a region.'
 
 
+class EndpointVariantError(BaseEndpointResolverError):
+    """
+    Could not construct modeled endpoint variant.
+
+    :ivar error_msg: The message explaining why the modeled endpoint variant
+        is unable to be constructed.
+
+    """
+
+    fmt = ('Unable to construct a modeled endpoint with the following '
+           'variant(s) {tags}: ')
+
+
 class UnknownEndpointError(BaseEndpointResolverError, ValueError):
     """
     Could not construct an endpoint.
@@ -198,6 +234,20 @@ class UnknownEndpointError(BaseEndpointResolverError, ValueError):
         '{service_name} in region {region_name}')
 
 
+class UnknownFIPSEndpointError(BaseEndpointResolverError):
+    """
+    Could not construct a FIPS endpoint.
+
+    :ivar service_name: The name of the service.
+    :ivar region_name: The name of the region.
+    """
+    fmt = (
+        'The provided FIPS pseudo-region "{region_name}" is not known for '
+        'the service "{service_name}". A FIPS compliant endpoint cannot be '
+        'constructed.'
+    )
+
+
 class ProfileNotFound(BotoCoreError):
     """
     The specified configuration profile was not found in the
@@ -500,12 +550,20 @@ class UnsupportedOutpostResourceError(BotoCoreError):
     )
 
 
+class UnsupportedS3ConfigurationError(BotoCoreError):
+    """Error when an unsupported configuration is used with access-points"""
+    fmt = (
+        'Unsupported configuration when using S3: {msg}'
+    )
+
+
 class UnsupportedS3AccesspointConfigurationError(BotoCoreError):
     """Error when an unsupported configuration is used with access-points"""
     fmt = (
         'Unsupported configuration when using S3 access-points: {msg}'
     )
 
+
 class InvalidEndpointDiscoveryConfigurationError(BotoCoreError):
     """Error when invalid value supplied for endpoint_discovery_enabled"""
     fmt = (
@@ -513,6 +571,7 @@ class InvalidEndpointDiscoveryConfigurationError(BotoCoreError):
         'Expected one of ("true", "false", "auto") but got {config_value}.'
     )
 
+
 class UnsupportedS3ControlConfigurationError(BotoCoreError):
     """Error when an unsupported configuration is used with S3 Control"""
     fmt = (
@@ -593,6 +652,10 @@ class MD5UnavailableError(BotoCoreError):
     fmt = "This system does not support MD5 generation."
 
 
+class MissingDependencyException(BotoCoreError):
+    fmt = "Missing Dependency: {msg}"
+
+
 class MetadataRetrievalError(BotoCoreError):
     fmt = "Error retrieving metadata: {error_msg}"
 
