@@ -18,7 +18,6 @@ This module contains builtin handlers for events emitted by botocore.
 
 import base64
 import logging
-import xml.etree.cElementTree
 import copy
 import re
 import warnings
@@ -26,7 +25,8 @@ import uuid
 
 from botocore.compat import (
     unquote, json, six, unquote_str, ensure_bytes, get_md5,
-    MD5_AVAILABLE, OrderedDict, urlsplit, urlunsplit, XMLParseError
+    OrderedDict, urlsplit, urlunsplit, XMLParseError,
+    ETree,
 )
 from botocore.docs.utils import AutoPopulatedParam
 from botocore.docs.utils import HideParamFromOperations
@@ -37,17 +37,22 @@ from botocore.signers import add_generate_db_auth_token
 from botocore.exceptions import ParamValidationError
 from botocore.exceptions import AliasConflictParameterError
 from botocore.exceptions import UnsupportedTLSVersionWarning
-from botocore.exceptions import MissingServiceIdError
 from botocore.utils import percent_encode, SAFE_CHARS
 from botocore.utils import switch_host_with_param
-from botocore.utils import hyphenize_service_id
+from botocore.utils import conditionally_calculate_md5
+from botocore.utils import is_global_accesspoint
 
-from botocore import retryhandler
 from botocore import utils
-from botocore import translate
 import botocore
 import botocore.auth
 
+# Keep these imported.  There's pre-existing code that uses them.
+from botocore import retryhandler # noqa
+from botocore import translate # noqa
+from botocore.compat import MD5_AVAILABLE # noqa
+from botocore.exceptions import MissingServiceIdError # noqa
+from botocore.utils import hyphenize_service_id # noqa
+
 
 logger = logging.getLogger(__name__)
 
@@ -59,10 +64,15 @@ REGISTER_LAST = object()
 # combination of uppercase letters, lowercase letters, numbers, periods
 # (.), hyphens (-), and underscores (_).
 VALID_BUCKET = re.compile(r'^[a-zA-Z0-9.\-_]{1,255}$')
-VALID_S3_ARN = re.compile(
-    r'^arn:(aws).*:s3:[a-z\-0-9]+:[0-9]{12}:accesspoint[/:]'
-    r'[a-zA-Z0-9\-]{1,63}$'
+_ACCESSPOINT_ARN = (
+    r'^arn:(aws).*:(s3|s3-object-lambda):[a-z\-0-9]*:[0-9]{12}:accesspoint[/:]'
+    r'[a-zA-Z0-9\-.]{1,63}$'
 )
+_OUTPOST_ARN = (
+    r'^arn:(aws).*:s3-outposts:[a-z\-0-9]+:[0-9]{12}:outpost[/:]'
+    r'[a-zA-Z0-9\-]{1,63}[/:]accesspoint[/:][a-zA-Z0-9\-]{1,63}$'
+)
+VALID_S3_ARN = re.compile('|'.join([_ACCESSPOINT_ARN, _OUTPOST_ARN]))
 VERSION_ID_SUFFIX = re.compile(r'\?versionId=[^\s]+$')
 
 SERVICE_NAME_ALIASES = {
@@ -74,6 +84,32 @@ def handle_service_name_alias(service_name, **kwargs):
     return SERVICE_NAME_ALIASES.get(service_name, service_name)
 
 
+def escape_xml_payload(params, **kwargs):
+    # Replace \r and \n with the escaped sequence over the whole XML document
+    # to avoid linebreak normalization modifying customer input when the
+    # document is parsed. Ideally, we would do this in ElementTree.tostring,
+    # but it doesn't allow us to override entity escaping for text fields. For
+    # this operation \r and \n can only appear in the XML document if they were
+    # passed as part of the customer input.
+    body = params['body']
+    replaced = False
+    if b'\r' in body:
+        replaced = True
+        body = body.replace(b'\r', b'&#xD;')
+    if b'\n' in body:
+        replaced = True
+        body = body.replace(b'\n', b'&#xA;')
+
+    if not replaced:
+        return
+
+    params['body'] = body
+    if 'Content-MD5' in params['headers']:
+        # The Content-MD5 is now wrong, so we'll need to recalculate it
+        del params['headers']['Content-MD5']
+        conditionally_calculate_md5(params, **kwargs)
+
+
 def check_for_200_error(response, **kwargs):
     # From: http://docs.aws.amazon.com/AmazonS3/latest/API/RESTObjectCOPY.html
     # There are two opportunities for a copy request to return an error. One
@@ -105,8 +141,8 @@ def check_for_200_error(response, **kwargs):
 def _looks_like_special_case_error(http_response):
     if http_response.status_code == 200:
         try:
-            parser = xml.etree.cElementTree.XMLParser(
-                target=xml.etree.cElementTree.TreeBuilder(),
+            parser = ETree.XMLParser(
+                target=ETree.TreeBuilder(),
                 encoding='utf-8')
             parser.feed(http_response.content)
             root = parser.close()
@@ -143,7 +179,10 @@ def set_operation_specific_signer(context, signing_name, **kwargs):
     if auth_type.startswith('v4'):
         signature_version = 'v4'
         if signing_name == 's3':
-            signature_version = 's3v4'
+            if is_global_accesspoint(context):
+                signature_version = 's3v4a'
+            else:
+                signature_version = 's3v4'
 
         # If the operation needs an unsigned body, we set additional context
         # allowing the signer to be aware of this.
@@ -192,38 +231,6 @@ def json_decode_template_body(parsed, **kwargs):
             logger.debug('error loading JSON', exc_info=True)
 
 
-def calculate_md5(params, **kwargs):
-    request_dict = params
-    if request_dict['body'] and 'Content-MD5' not in params['headers']:
-        body = request_dict['body']
-        if isinstance(body, (bytes, bytearray)):
-            binary_md5 = _calculate_md5_from_bytes(body)
-        else:
-            binary_md5 = _calculate_md5_from_file(body)
-        base64_md5 = base64.b64encode(binary_md5).decode('ascii')
-        params['headers']['Content-MD5'] = base64_md5
-
-
-def _calculate_md5_from_bytes(body_bytes):
-    md5 = get_md5(body_bytes)
-    return md5.digest()
-
-
-def _calculate_md5_from_file(fileobj):
-    start_position = fileobj.tell()
-    md5 = get_md5()
-    for chunk in iter(lambda: fileobj.read(1024 * 1024), b''):
-        md5.update(chunk)
-    fileobj.seek(start_position)
-    return md5.digest()
-
-
-def conditionally_calculate_md5(params, context, request_signer, **kwargs):
-    """Only add a Content-MD5 if the system supports it."""
-    if MD5_AVAILABLE:
-        calculate_md5(params, **kwargs)
-
-
 def validate_bucket_name(params, **kwargs):
     if 'Bucket' not in params:
         return
@@ -273,8 +280,10 @@ def _sse_md5(params, sse_member_prefix='SSECustomer'):
 
 
 def _needs_s3_sse_customization(params, sse_member_prefix):
-    return (params.get(sse_member_prefix + 'Key') is not None and
-            sse_member_prefix + 'KeyMD5' not in params)
+    return (
+        params.get(sse_member_prefix + 'Key') is not None
+        and sse_member_prefix + 'KeyMD5' not in params
+    )
 
 
 def disable_signing(**kwargs):
@@ -338,6 +347,12 @@ def document_copy_source_form(section, event_name, **kwargs):
             "the string format because it is more explicit.  The dictionary "
             "format is: {'Bucket': 'bucket', 'Key': 'key', 'VersionId': 'id'}."
             "  Note that the VersionId key is optional and may be omitted."
+            " To specify an S3 access point, provide the access point"
+            " ARN for the ``Bucket`` key in the copy source dictionary. If you"
+            " want to provide the copy source for an S3 access point as a"
+            " string instead of a dictionary, the ARN provided must be the"
+            " full S3 access point object ARN"
+            " (i.e. {accesspoint_arn}/object/{key})"
         )
 
 
@@ -371,12 +386,16 @@ def handle_copy_source_param(params, **kwargs):
 def _quote_source_header_from_dict(source_dict):
     try:
         bucket = source_dict['Bucket']
-        key = percent_encode(source_dict['Key'], safe=SAFE_CHARS + '/')
+        key = source_dict['Key']
         version_id = source_dict.get('VersionId')
+        if VALID_S3_ARN.search(bucket):
+            final = '%s/object/%s' % (bucket, key)
+        else:
+            final = '%s/%s' % (bucket, key)
     except KeyError as e:
         raise ParamValidationError(
             report='Missing required parameter: %s' % str(e))
-    final = '%s/%s' % (bucket, key)
+    final = percent_encode(final, safe=SAFE_CHARS + '/')
     if version_id is not None:
         final += '?versionId=%s' % version_id
     return final
@@ -495,8 +514,8 @@ def parse_get_bucket_location(parsed, http_response, **kwargs):
     if http_response.raw is None:
         return
     response_body = http_response.content
-    parser = xml.etree.cElementTree.XMLParser(
-        target=xml.etree.cElementTree.TreeBuilder(),
+    parser = ETree.XMLParser(
+        target=ETree.TreeBuilder(),
         encoding='utf-8')
     parser.feed(response_body)
     root = parser.close()
@@ -543,7 +562,7 @@ def validate_ascii_metadata(params, **kwargs):
         try:
             key.encode('ascii')
             value.encode('ascii')
-        except UnicodeEncodeError as e:
+        except UnicodeEncodeError:
             error_msg = (
                 'Non ascii characters found in S3 metadata '
                 'for key "%s", value: "%s".  \nS3 metadata can only '
@@ -750,8 +769,7 @@ def decode_list_object_versions(parsed, context, **kwargs):
 
 
 def _decode_list_object(top_level_keys, nested_keys, parsed, context):
-    if parsed.get('EncodingType') == 'url' and \
-                    context.get('encoding_type_auto_set'):
+    if parsed.get('EncodingType') == 'url' and context.get('encoding_type_auto_set'):
         # URL decode top-level keys in the response if present.
         for key in top_level_keys:
             if key in parsed:
@@ -865,6 +883,7 @@ class ClientMethodAlias(object):
         return getattr(client, self._actual)
 
 
+# TODO: Remove this class as it is no longer used
 class HeaderToHostHoister(object):
     """Takes a header and moves it to the front of the hoststring.
     """
@@ -918,6 +937,12 @@ def inject_api_version_header_if_needed(model, params, **kwargs):
     params['headers']['x-amz-api-version'] = model.service_model.api_version
 
 
+def remove_lex_v2_start_conversation(class_attributes, **kwargs):
+    """Operation requires h2 which is currently unsupported in Python"""
+    if 'start_conversation' in class_attributes:
+        del class_attributes['start_conversation']
+
+
 # This is a list of (event_name, handler).
 # When a Session is created, everything in this list will be
 # automatically registered with that Session.
@@ -933,6 +958,7 @@ BUILTIN_HANDLERS = [
     ('creating-client-class', add_generate_presigned_url),
     ('creating-client-class.s3', add_generate_presigned_post),
     ('creating-client-class.iot-data', check_openssl_supports_tls_version_1_2),
+    ('creating-client-class.lex-runtime-v2', remove_lex_v2_start_conversation),
     ('after-call.iam', json_decode_policies),
 
     ('after-call.ec2.GetConsoleOutput', decode_console_output),
@@ -949,26 +975,6 @@ BUILTIN_HANDLERS = [
      set_list_objects_encoding_type_url),
     ('before-parameter-build.s3.ListObjectVersions',
      set_list_objects_encoding_type_url),
-    ('before-call.s3.PutBucketTagging', calculate_md5),
-    ('before-call.s3.PutBucketLifecycle', calculate_md5),
-    ('before-call.s3.PutBucketLifecycleConfiguration', calculate_md5),
-    ('before-call.s3.PutBucketCors', calculate_md5),
-    ('before-call.s3.DeleteObjects', calculate_md5),
-    ('before-call.s3.PutBucketReplication', calculate_md5),
-    ('before-call.s3.PutObject', conditionally_calculate_md5),
-    ('before-call.s3.UploadPart', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketAcl', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketLogging', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketNotification', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketPolicy', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketRequestPayment', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketVersioning', conditionally_calculate_md5),
-    ('before-call.s3.PutBucketWebsite', conditionally_calculate_md5),
-    ('before-call.s3.PutObjectAcl', conditionally_calculate_md5),
-    ('before-call.s3.PutObjectLegalHold', calculate_md5),
-    ('before-call.s3.PutObjectRetention', calculate_md5),
-    ('before-call.s3.PutObjectLockConfiguration', calculate_md5),
-
     ('before-parameter-build.s3.CopyObject',
      handle_copy_source_param),
     ('before-parameter-build.s3.UploadPartCopy',
@@ -983,6 +989,10 @@ BUILTIN_HANDLERS = [
     ('before-call.s3', add_expect_header),
     ('before-call.glacier', add_glacier_version),
     ('before-call.apigateway', add_accept_header),
+    ('before-call.s3.PutObject', conditionally_calculate_md5),
+    ('before-call.s3.UploadPart', conditionally_calculate_md5),
+    ('before-call.s3.DeleteObjects', escape_xml_payload),
+    ('before-call.s3.PutBucketLifecycleConfiguration', escape_xml_payload),
     ('before-call.glacier.UploadArchive', add_glacier_checksums),
     ('before-call.glacier.UploadMultipartPart', add_glacier_checksums),
     ('before-call.ec2.CopySnapshot', inject_presigned_url_ec2),
@@ -1051,8 +1061,7 @@ BUILTIN_HANDLERS = [
      AutoPopulatedParam('SSECustomerKeyMD5').document_auto_populated_param),
     # S3 SSE Copy Source documentation modifications
     ('docs.*.s3.*.complete-section',
-     AutoPopulatedParam(
-        'CopySourceSSECustomerKeyMD5').document_auto_populated_param),
+     AutoPopulatedParam('CopySourceSSECustomerKeyMD5').document_auto_populated_param),
     # Add base64 information to Lambda
     ('docs.*.lambda.UpdateFunctionCode.complete-section',
      document_base64_encoding('ZipFile')),
@@ -1079,6 +1088,8 @@ BUILTIN_HANDLERS = [
      inject_presigned_url_rds),
     ('before-call.rds.CreateDBInstanceReadReplica',
      inject_presigned_url_rds),
+    ('before-call.rds.StartDBInstanceAutomatedBackupsReplication',
+     inject_presigned_url_rds),
 
     # RDS PresignedUrl documentation customizations
     ('docs.*.rds.CopyDBClusterSnapshot.complete-section',
@@ -1089,6 +1100,8 @@ BUILTIN_HANDLERS = [
      AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
     ('docs.*.rds.CreateDBInstanceReadReplica.complete-section',
      AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
+    ('docs.*.rds.StartDBInstanceAutomatedBackupsReplication.complete-section',
+     AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
 
     #############
     # Neptune
@@ -1098,24 +1111,31 @@ BUILTIN_HANDLERS = [
     ('before-call.neptune.CreateDBCluster',
      inject_presigned_url_rds),
 
-    # RDS PresignedUrl documentation customizations
+    # Neptune PresignedUrl documentation customizations
     ('docs.*.neptune.CopyDBClusterSnapshot.complete-section',
      AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
     ('docs.*.neptune.CreateDBCluster.complete-section',
      AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
 
     #############
-    # S3 Control
+    # DocDB
     #############
-    ('before-call.s3-control.*',
-     HeaderToHostHoister('x-amz-account-id').hoist),
+    ('before-call.docdb.CopyDBClusterSnapshot',
+     inject_presigned_url_rds),
+    ('before-call.docdb.CreateDBCluster',
+     inject_presigned_url_rds),
+
+    # DocDB PresignedUrl documentation customizations
+    ('docs.*.docdb.CopyDBClusterSnapshot.complete-section',
+     AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
+    ('docs.*.docdb.CreateDBCluster.complete-section',
+     AutoPopulatedParam('PreSignedUrl').document_auto_populated_param),
 
     ###########
     # SMS Voice
-     ##########
+    ###########
     ('docs.title.sms-voice',
-     DeprecatedServiceDocumenter(
-         'pinpoint-sms-voice').inject_deprecation_notice),
+     DeprecatedServiceDocumenter('pinpoint-sms-voice').inject_deprecation_notice),
     ('before-call', inject_api_version_header_if_needed),
 
 ]
