@@ -49,6 +49,8 @@ from botocore.compat import json, formatdate
 from botocore.utils import parse_to_aware_datetime
 from botocore.utils import percent_encode
 from botocore.utils import is_json_value_header
+from botocore.utils import conditionally_calculate_md5
+from botocore.utils import has_header
 from botocore import validate
 
 
@@ -184,6 +186,12 @@ class Serializer(object):
 
         return host_prefix_expression.format(**format_kwargs)
 
+    def _prepare_additional_traits(self, request, operation_model):
+        """Determine if additional traits are required for given model"""
+        if operation_model.http_checksum_required:
+            conditionally_calculate_md5(request)
+        return request
+
 
 class QuerySerializer(Serializer):
 
@@ -210,6 +218,9 @@ class QuerySerializer(Serializer):
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(
+            serialized, operation_model
+        )
         return serialized
 
     def _serialize(self, serialized, value, shape, prefix=''):
@@ -343,6 +354,9 @@ class JSONSerializer(Serializer):
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(
+            serialized, operation_model
+        )
         return serialized
 
     def _serialize(self, serialized, value, shape, key=None):
@@ -351,21 +365,24 @@ class JSONSerializer(Serializer):
         method(serialized, value, shape, key)
 
     def _serialize_type_structure(self, serialized, value, shape, key):
-        if key is not None:
-            # If a key is provided, this is a result of a recursive
-            # call so we need to add a new child dict as the value
-            # of the passed in serialized dict.  We'll then add
-            # all the structure members as key/vals in the new serialized
-            # dictionary we just created.
-            new_serialized = self.MAP_TYPE()
-            serialized[key] = new_serialized
-            serialized = new_serialized
-        members = shape.members
-        for member_key, member_value in value.items():
-            member_shape = members[member_key]
-            if 'name' in member_shape.serialization:
-                member_key = member_shape.serialization['name']
-            self._serialize(serialized, member_value, member_shape, member_key)
+        if shape.is_document_type:
+            serialized[key] = value
+        else:
+            if key is not None:
+                # If a key is provided, this is a result of a recursive
+                # call so we need to add a new child dict as the value
+                # of the passed in serialized dict.  We'll then add
+                # all the structure members as key/vals in the new serialized
+                # dictionary we just created.
+                new_serialized = self.MAP_TYPE()
+                serialized[key] = new_serialized
+                serialized = new_serialized
+            members = shape.members
+            for member_key, member_value in value.items():
+                member_shape = members[member_key]
+                if 'name' in member_shape.serialization:
+                    member_key = member_shape.serialization['name']
+                self._serialize(serialized, member_value, member_shape, member_key)
 
     def _serialize_type_map(self, serialized, value, shape, key):
         map_obj = self.MAP_TYPE()
@@ -455,11 +472,15 @@ class BaseRestSerializer(Serializer):
             serialized['headers'] = partitioned['headers']
         self._serialize_payload(partitioned, parameters,
                                 serialized, shape, shape_members)
+        self._serialize_content_type(serialized, shape, shape_members)
 
         host_prefix = self._expand_host_prefix(parameters, operation_model)
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(
+            serialized, operation_model
+        )
         return serialized
 
     def _render_uri_template(self, uri_template, params):
@@ -487,8 +508,7 @@ class BaseRestSerializer(Serializer):
         # shape - Describes the expected input shape
         # shape_members - The members of the input struct shape
         payload_member = shape.serialization.get('payload')
-        if payload_member is not None and \
-                shape_members[payload_member].type_name in ['blob', 'string']:
+        if self._has_streaming_payload(payload_member, shape_members):
             # If it's streaming, then the body is just the
             # value of the payload.
             body_payload = parameters.get(payload_member, b'')
@@ -502,9 +522,38 @@ class BaseRestSerializer(Serializer):
                 serialized['body'] = self._serialize_body_params(
                     body_params,
                     shape_members[payload_member])
+            else:
+                serialized['body'] = self._serialize_empty_body()
         elif partitioned['body_kwargs']:
             serialized['body'] = self._serialize_body_params(
                 partitioned['body_kwargs'], shape)
+        elif self._requires_empty_body(shape):
+            serialized['body'] = self._serialize_empty_body()
+
+    def _serialize_empty_body(self):
+        return b''
+
+    def _serialize_content_type(self, serialized, shape, shape_members):
+        """
+        Some protocols require varied Content-Type headers
+        depending on user input. This allows subclasses to apply
+        this conditionally.
+        """
+        pass
+
+    def _requires_empty_body(self, shape):
+        """
+        Some protocols require a specific body to represent an empty
+        payload. This allows subclasses to apply this conditionally.
+        """
+        return False
+
+    def _has_streaming_payload(self, payload, shape_members):
+        """Determine if payload is streaming (a blob or string)."""
+        return (
+            payload is not None and
+            shape_members[payload].type_name in ['blob', 'string']
+        )
 
     def _encode_payload(self, body):
         if isinstance(body, six.text_type):
@@ -526,15 +575,15 @@ class BaseRestSerializer(Serializer):
             if isinstance(param_value, dict):
                 partitioned['query_string_kwargs'].update(param_value)
             elif isinstance(param_value, bool):
-                partitioned['query_string_kwargs'][
-                    key_name] = str(param_value).lower()
+                bool_str = str(param_value).lower()
+                partitioned['query_string_kwargs'][key_name] = bool_str
             elif member.type_name == 'timestamp':
                 timestamp_format = member.serialization.get(
                     'timestampFormat', self.QUERY_STRING_TIMESTAMP_FORMAT)
-                partitioned['query_string_kwargs'][
-                    key_name] = self._convert_timestamp_to_str(
-                        param_value, timestamp_format
-                    )
+                timestamp = self._convert_timestamp_to_str(
+                    param_value, timestamp_format
+                )
+                partitioned['query_string_kwargs'][key_name] = timestamp
             else:
                 partitioned['query_string_kwargs'][key_name] = param_value
         elif location == 'header':
@@ -570,6 +619,12 @@ class BaseRestSerializer(Serializer):
             timestamp_format = shape.serialization.get(
                 'timestampFormat', self.HEADER_TIMESTAMP_FORMAT)
             return self._convert_timestamp_to_str(timestamp, timestamp_format)
+        elif shape.type_name == 'list':
+            converted_value = [
+                self._convert_header_value(shape.member, v)
+                for v in value if v is not None
+            ]
+            return ",".join(converted_value)
         elif is_json_value_header(shape):
             # Serialize with no spaces after separators to save space in
             # the header.
@@ -580,6 +635,31 @@ class BaseRestSerializer(Serializer):
 
 class RestJSONSerializer(BaseRestSerializer, JSONSerializer):
 
+    def _serialize_empty_body(self):
+        return b'{}'
+
+    def _requires_empty_body(self, shape):
+        """
+        Serialize an empty JSON object whenever the shape has
+        members not targeting a location.
+        """
+        for member, val in shape.members.items():
+            if 'location' not in val.serialization:
+                return True
+        return False
+
+    def _serialize_content_type(self, serialized, shape, shape_members):
+        """Set Content-Type to application/json for all structured bodies."""
+        payload = shape.serialization.get('payload')
+        if self._has_streaming_payload(payload, shape_members):
+            # Don't apply content-type to streaming bodies
+            return
+
+        has_body = serialized['body'] != b''
+        has_content_type = has_header('Content-Type', serialized['headers'])
+        if has_body and not has_content_type:
+            serialized['headers']['Content-Type'] = 'application/json'
+
     def _serialize_body_params(self, params, shape):
         serialized_body = self.MAP_TYPE()
         self._serialize(serialized_body, params, shape)
