@@ -10,10 +10,9 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-import mock
 from datetime import datetime
 
-from tests import BaseSessionTest, ClientHTTPStubber
+from tests import mock, BaseSessionTest, ClientHTTPStubber
 
 
 class TestLex(BaseSessionTest):
@@ -34,8 +33,8 @@ class TestLex(BaseSessionTest):
 
         timestamp = datetime(2017, 3, 22, 0, 0)
 
-        with mock.patch('botocore.auth.datetime') as _datetime:
-            _datetime.datetime.utcnow.return_value = timestamp
+        with mock.patch('botocore.auth.datetime.datetime') as _datetime:
+            _datetime.utcnow.return_value = timestamp
             self.http_stubber.add_response(body=b'{}')
             with self.http_stubber:
                 self.client.post_content(**params)
@@ -58,3 +57,19 @@ class TestLex(BaseSessionTest):
 
         content_header = request.headers.get('x-amz-content-sha256')
         self.assertEqual(content_header, b'UNSIGNED-PAYLOAD')
+
+
+class TestLexV2(BaseSessionTest):
+
+    def test_start_conversation(self):
+        """StartConversation operation removed due to h2 requirement"""
+        lexv2 = self.session.create_client('lexv2-runtime', 'us-west-2')
+        try:
+            lexv2.start_conversation
+        except AttributeError:
+            pass
+        else:
+            self.fail(
+                'start_conversation shouldn\'t be available on the '
+                'lexv2-runtime client.'
+            )
