@@ -11,19 +11,19 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import socket
+
+import pytest
+
+from tests import mock
 from tests import unittest
 
-from mock import Mock, patch, sentinel
-
 from botocore.compat import six
-from botocore.awsrequest import AWSRequest
 from botocore.endpoint import Endpoint, DEFAULT_TIMEOUT
 from botocore.endpoint import EndpointCreator
-from botocore.exceptions import EndpointConnectionError
-from botocore.exceptions import ConnectionClosedError
 from botocore.exceptions import HTTPClientError
 from botocore.httpsession import URLLib3Session
 from botocore.model import OperationModel, ServiceId
+from botocore.model import ServiceModel, StructureShape
 
 
 def request_dict(**kwargs):
@@ -53,21 +53,21 @@ class RecordStreamResets(six.StringIO):
 class TestEndpointBase(unittest.TestCase):
 
     def setUp(self):
-        self.op = Mock()
+        self.op = mock.Mock()
         self.op.has_streaming_output = False
         self.op.has_event_stream_output = False
         self.op.metadata = {'protocol': 'json'}
-        self.event_emitter = Mock()
+        self.event_emitter = mock.Mock()
         self.event_emitter.emit.return_value = []
-        self.factory_patch = patch(
+        self.factory_patch = mock.patch(
             'botocore.parsers.ResponseParserFactory')
         self.factory = self.factory_patch.start()
         self.endpoint = Endpoint(
             'https://ec2.us-west-2.amazonaws.com/',
             endpoint_prefix='ec2',
             event_emitter=self.event_emitter)
-        self.http_session = Mock()
-        self.http_session.send.return_value = Mock(
+        self.http_session = mock.Mock()
+        self.http_session.send.return_value = mock.Mock(
             status_code=200, headers={}, content=b'{"Foo": "bar"}',
         )
         self.endpoint.http_session = self.http_session
@@ -119,17 +119,56 @@ class TestEndpointFeatures(TestEndpointBase):
     def test_make_request_with_context(self):
         r = request_dict()
         r['context'] = {'signing': {'region': 'us-west-2'}}
-        with patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
+        with mock.patch('botocore.endpoint.Endpoint.prepare_request') as prepare:
             self.endpoint.make_request(self.op, r)
         request = prepare.call_args[0][0]
         self.assertEqual(request.context['signing']['region'], 'us-west-2')
 
+    def test_parses_modeled_exception_fields(self):
+        # Setup the service model to have exceptions to generate the mapping
+        self.service_model = mock.Mock(spec=ServiceModel)
+        self.op.service_model = self.service_model
+        self.exception_shape = mock.Mock(spec=StructureShape)
+        shape_for_error_code = self.service_model.shape_for_error_code
+        shape_for_error_code.return_value = self.exception_shape
+
+        r = request_dict()
+        self.http_session.send.return_value = mock.Mock(
+            status_code=400, headers={}, content=b'',
+        )
+        parser = mock.Mock()
+        parser.parse.side_effect = [
+            {
+                'Error': {
+                    'Code': 'ExceptionShape',
+                    'Message': 'Some message',
+                }
+            },
+            {'SomeField': 'Foo'},
+        ]
+        self.factory.return_value.create_parser.return_value = parser
+        _, response = self.endpoint.make_request(self.op, r)
+        # The parser should be called twice, once for the original
+        # error parse and once again for the modeled exception parse
+        self.assertEqual(parser.parse.call_count, 2)
+        parse_calls = parser.parse.call_args_list
+        self.assertEqual(parse_calls[1][0][1], self.exception_shape)
+        self.assertEqual(parse_calls[0][0][1], self.op.output_shape)
+        expected_response = {
+            'Error': {
+                'Code': 'ExceptionShape',
+                'Message': 'Some message',
+            },
+            'SomeField': 'Foo',
+        }
+        self.assertEqual(response, expected_response)
+
 
 class TestRetryInterface(TestEndpointBase):
     def setUp(self):
         super(TestRetryInterface, self).setUp()
         self.retried_on_exception = None
-        self._operation = Mock(spec=OperationModel)
+        self._operation = mock.Mock(spec=OperationModel)
         self._operation.name = 'DescribeInstances'
         self._operation.metadata = {'protocol': 'query'}
         self._operation.service_model.service_id = ServiceId('EC2')
@@ -179,7 +218,7 @@ class TestRetryInterface(TestEndpointBase):
     def test_retry_attempts_added_to_response_metadata(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
             num_retries=1)
-        parser = Mock()
+        parser = mock.Mock()
         parser.parse.return_value = {'ResponseMetadata': {}}
         self.factory.return_value.create_parser.return_value = parser
         response = self.endpoint.make_request(self._operation, request_dict())
@@ -188,7 +227,7 @@ class TestRetryInterface(TestEndpointBase):
     def test_retry_attempts_is_zero_when_not_retried(self):
         self.event_emitter.emit.side_effect = self.get_emitter_responses(
             num_retries=0)
-        parser = Mock()
+        parser = mock.Mock()
         parser.parse.return_value = {'ResponseMetadata': {}}
         self.factory.return_value.create_parser.return_value = parser
         response = self.endpoint.make_request(self._operation, request_dict())
@@ -210,7 +249,7 @@ class TestS3ResetStreamOnRetry(TestEndpointBase):
             return 0
 
     def test_reset_stream_on_retry(self):
-        op = Mock()
+        op = mock.Mock()
         body = RecordStreamResets('foobar')
         op.name = 'PutObject'
         op.has_streaming_output = True
@@ -238,14 +277,14 @@ class TestEventStreamBody(TestEndpointBase):
 
 class TestEndpointCreator(unittest.TestCase):
     def setUp(self):
-        self.service_model = Mock(
+        self.service_model = mock.Mock(
             endpoint_prefix='ec2', signature_version='v2',
             signing_name='ec2')
         self.environ = {}
-        self.environ_patch = patch('os.environ', self.environ)
+        self.environ_patch = mock.patch('os.environ', self.environ)
         self.environ_patch.start()
-        self.creator = EndpointCreator(Mock())
-        self.mock_session = Mock(spec=URLLib3Session)
+        self.creator = EndpointCreator(mock.Mock())
+        self.mock_session = mock.Mock(spec=URLLib3Session)
 
     def tearDown(self):
         self.environ_patch.stop()
@@ -256,8 +295,27 @@ class TestEndpointCreator(unittest.TestCase):
             endpoint_url='https://endpoint.url')
         self.assertEqual(endpoint.host, 'https://endpoint.url')
 
-    def test_create_endpoint_with_default_timeout(self):
+    def test_creates_endpoint_with_ipv4_url(self):
         endpoint = self.creator.create_endpoint(
+            self.service_model, region_name='us-east-1',
+            endpoint_url='https://192.168.0.0')
+        self.assertEqual(endpoint.host, 'https://192.168.0.0')
+
+    def test_creates_endpoint_with_ipv6_url(self):
+        endpoint = self.creator.create_endpoint(
+            self.service_model, region_name='us-east-1',
+            endpoint_url='https://[100:0:2::61]:7480')
+        self.assertEqual(endpoint.host, 'https://[100:0:2::61]:7480')
+
+    def test_raises_error_with_invalid_url(self):
+        with pytest.raises(ValueError):
+            self.creator.create_endpoint(
+                self.service_model, region_name='us-east-1',
+                endpoint_url='https://*.aws.amazon.com/'
+            )
+
+    def test_create_endpoint_with_default_timeout(self):
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com',
             http_session_cls=self.mock_session)
@@ -265,7 +323,7 @@ class TestEndpointCreator(unittest.TestCase):
         self.assertEqual(session_args.get('timeout'), DEFAULT_TIMEOUT)
 
     def test_create_endpoint_with_customized_timeout(self):
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com', timeout=123,
             http_session_cls=self.mock_session)
@@ -273,7 +331,7 @@ class TestEndpointCreator(unittest.TestCase):
         self.assertEqual(session_args.get('timeout'), 123)
 
     def test_get_endpoint_default_verify_ssl(self):
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com',
             http_session_cls=self.mock_session)
@@ -281,7 +339,7 @@ class TestEndpointCreator(unittest.TestCase):
         self.assertTrue(session_args.get('verify'))
 
     def test_verify_ssl_can_be_disabled(self):
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com', verify=False,
             http_session_cls=self.mock_session)
@@ -289,7 +347,7 @@ class TestEndpointCreator(unittest.TestCase):
         self.assertFalse(session_args.get('verify'))
 
     def test_verify_ssl_can_specify_cert_bundle(self):
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com', verify='/path/cacerts.pem',
             http_session_cls=self.mock_session)
@@ -298,7 +356,7 @@ class TestEndpointCreator(unittest.TestCase):
 
     def test_client_cert_can_specify_path(self):
         client_cert = '/some/path/cert'
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com', client_cert=client_cert,
             http_session_cls=self.mock_session)
@@ -307,7 +365,7 @@ class TestEndpointCreator(unittest.TestCase):
 
     def test_honor_cert_bundle_env_var(self):
         self.environ['REQUESTS_CA_BUNDLE'] = '/env/cacerts.pem'
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com',
             http_session_cls=self.mock_session)
@@ -316,7 +374,7 @@ class TestEndpointCreator(unittest.TestCase):
 
     def test_env_ignored_if_explicitly_passed(self):
         self.environ['REQUESTS_CA_BUNDLE'] = '/env/cacerts.pem'
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com', verify='/path/cacerts.pem',
             http_session_cls=self.mock_session)
@@ -325,7 +383,7 @@ class TestEndpointCreator(unittest.TestCase):
         self.assertEqual(session_args.get('verify'), '/path/cacerts.pem')
 
     def test_can_specify_max_pool_conns(self):
-        endpoint = self.creator.create_endpoint(
+        self.creator.create_endpoint(
             self.service_model, region_name='us-west-2',
             endpoint_url='https://example.com',
             max_pool_connections=100,
