@@ -11,10 +11,9 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
-from tests import unittest, BaseSessionTest
+from tests import mock, unittest, BaseSessionTest
 
 import base64
-import mock
 import copy
 import os
 import json
@@ -24,7 +23,6 @@ import botocore.session
 from botocore.compat import OrderedDict
 from botocore.exceptions import ParamValidationError, MD5UnavailableError
 from botocore.exceptions import AliasConflictParameterError
-from botocore.exceptions import MissingServiceIdError
 from botocore.awsrequest import AWSRequest
 from botocore.compat import quote, six
 from botocore.config import Config
@@ -32,10 +30,8 @@ from botocore.docs.bcdoc.restdoc import DocumentStructure
 from botocore.docs.params import RequestParamsDocumenter
 from botocore.docs.example import RequestExampleDocumenter
 from botocore.hooks import HierarchicalEmitter
-from botocore.loaders import Loader
 from botocore.model import OperationModel, ServiceModel, ServiceId
 from botocore.model import DenormalizedStructureBuilder
-from botocore.session import Session
 from botocore.signers import RequestSigner
 from botocore.credentials import Credentials
 from botocore.utils import conditionally_calculate_md5
@@ -126,7 +122,7 @@ class TestHandlers(BaseSessionTest):
                          'foo/keyname%2B?versionId=asdf+')
 
     def test_copy_source_has_validation_failure(self):
-        with self.assertRaisesRegexp(ParamValidationError, 'Key'):
+        with self.assertRaisesRegex(ParamValidationError, 'Key'):
             handlers.handle_copy_source_param(
                 {'CopySource': {'Bucket': 'foo'}})
 
@@ -762,6 +758,16 @@ class TestHandlers(BaseSessionTest):
         except ParamValidationError:
             self.fail('The s3 arn: %s should pass validation' % arn)
 
+    def test_validation_is_s3_outpost_arn(self):
+        try:
+            arn = (
+                'arn:aws:s3-outposts:us-west-2:123456789012:outpost:'
+                'op-01234567890123456:accesspoint:myaccesspoint'
+            )
+            handlers.validate_bucket_name({'Bucket': arn})
+        except ParamValidationError:
+            self.fail('The s3 arn: %s should pass validation' % arn)
+
     def test_validation_is_global_s3_bucket_arn(self):
         with self.assertRaises(ParamValidationError):
             arn = 'arn:aws:s3:::mybucket'
@@ -1207,6 +1213,17 @@ class TestAddMD5(BaseMD5Test):
             request_dict['headers']['Content-MD5'],
             'OFj2IjCsPJFfMAxmQxLGPw==')
 
+    def test_add_md5_with_empty_body(self):
+        request_dict = {
+            'body': b'',
+            'headers': {}
+        }
+        self.md5_digest.return_value = b'8X\xf6"0\xac<\x91_0\x0cfC\x12\xc6?'
+        conditionally_calculate_md5(request_dict)
+        self.assertEqual(
+            request_dict['headers']['Content-MD5'],
+            'OFj2IjCsPJFfMAxmQxLGPw==')
+
     def test_add_md5_with_bytearray_object(self):
         request_dict = {
             'body': bytearray(b'foobar'),
@@ -1284,10 +1301,10 @@ class TestParameterAlias(unittest.TestCase):
             self.sample_section
         )
         contents = self.sample_section.flush_structure().decode('utf-8')
-        self.assertIn(':type ' + self.alias_name + ':',  contents)
-        self.assertIn(':param ' + self.alias_name + ':',  contents)
-        self.assertNotIn(':type ' + self.original_name + ':',  contents)
-        self.assertNotIn(':param ' + self.original_name + ':',  contents)
+        self.assertIn(':type ' + self.alias_name + ':', contents)
+        self.assertIn(':param ' + self.alias_name + ':', contents)
+        self.assertNotIn(':type ' + self.original_name + ':', contents)
+        self.assertNotIn(':param ' + self.original_name + ':', contents)
 
     def test_alias_parameter_in_documentation_request_example(self):
         RequestExampleDocumenter(
@@ -1298,7 +1315,7 @@ class TestParameterAlias(unittest.TestCase):
             self.sample_section
         )
         contents = self.sample_section.flush_structure().decode('utf-8')
-        self.assertIn(self.alias_name + '=',  contents)
+        self.assertIn(self.alias_name + '=', contents)
         self.assertNotIn(self.original_name + '=', contents)
 
 
@@ -1346,10 +1363,8 @@ class TestPrependToHost(unittest.TestCase):
 
     def test_does_validate_long_host(self):
         with self.assertRaises(ParamValidationError):
-           self._prepend_to_host(
-               'https://example.com/path', 'toolong'*100)
+            self._prepend_to_host('https://example.com/path', 'toolong' * 100)
 
     def test_does_validate_host_with_illegal_char(self):
         with self.assertRaises(ParamValidationError):
-           self._prepend_to_host(
-               'https://example.com/path', 'host#name')
+            self._prepend_to_host('https://example.com/path', 'host#name')
