@@ -1,34 +1,16 @@
+import pytest
+
 from tests import unittest
 
 from botocore import model
 from botocore.compat import OrderedDict
-from botocore.exceptions import MissingServiceIdError
 
 
-def test_missing_model_attribute_raises_exception():
-    # We're using a nose test generator here to cut down
-    # on the duplication.  The property names below
-    # all have the same test logic.
+@pytest.mark.parametrize("property_name", ['api_version', 'protocol'])
+def test_missing_model_attribute_raises_exception(property_name):
     service_model = model.ServiceModel({'metadata': {'endpointPrefix': 'foo'}})
-    property_names = ['api_version', 'protocol']
-
-    def _test_attribute_raise_exception(attr_name):
-        try:
-            getattr(service_model, attr_name)
-        except model.UndefinedModelAttributeError:
-            # This is what we expect, so the test passes.
-            pass
-        except Exception as e:
-            raise AssertionError("Expected UndefinedModelAttributeError to "
-                                 "be raised, but %s was raised instead" %
-                                 (e.__class__))
-        else:
-            raise AssertionError(
-                "Expected UndefinedModelAttributeError to "
-                "be raised, but no exception was raised for: %s" % attr_name)
-
-    for name in property_names:
-        yield _test_attribute_raise_exception, name
+    with pytest.raises(model.UndefinedModelAttributeError):
+        getattr(service_model, property_name)
 
 
 class TestServiceId(unittest.TestCase):
@@ -54,6 +36,21 @@ class TestServiceModel(unittest.TestCase):
                 'StringShape': {'type': 'string'}
             }
         }
+        self.error_shapes = {
+            'ExceptionOne': {
+                'exception': True,
+                'type': 'structure',
+                'members': {},
+            },
+            'ExceptionTwo': {
+                'exception': True,
+                'type': 'structure',
+                'members': {},
+                'error': {
+                    'code': 'FooCode'
+                }
+            },
+        }
         self.service_model = model.ServiceModel(self.model)
 
     def test_metadata_available(self):
@@ -90,8 +87,8 @@ class TestServiceModel(unittest.TestCase):
         }
         service_name = 'myservice'
         service_model = model.ServiceModel(service_model, service_name)
-        with self.assertRaisesRegexp(model.UndefinedModelAttributeError,
-                                     service_name):
+        with self.assertRaisesRegex(model.UndefinedModelAttributeError,
+                                    service_name):
             service_model.service_id
 
     def test_operation_does_not_exist(self):
@@ -112,6 +109,23 @@ class TestServiceModel(unittest.TestCase):
         self.assertEqual(repr(self.service_model),
                          'ServiceModel(endpoint-prefix)')
 
+    def test_shape_for_error_code(self):
+        self.model['shapes'].update(self.error_shapes)
+        self.service_model = model.ServiceModel(self.model)
+        shape = self.service_model.shape_for_error_code('ExceptionOne')
+        self.assertEqual(shape.name, 'ExceptionOne')
+        shape = self.service_model.shape_for_error_code('FooCode')
+        self.assertEqual(shape.name, 'ExceptionTwo')
+
+    def test_error_shapes(self):
+        self.model['shapes'].update(self.error_shapes)
+        self.service_model = model.ServiceModel(self.model)
+        error_shapes = self.service_model.error_shapes
+        error_shape_names = [shape.name for shape in error_shapes]
+        self.assertEqual(len(error_shape_names), 2)
+        self.assertIn('ExceptionOne', error_shape_names)
+        self.assertIn('ExceptionTwo', error_shape_names)
+
 
 class TestOperationModelFromService(unittest.TestCase):
     def setUp(self):
@@ -149,6 +163,36 @@ class TestOperationModelFromService(unittest.TestCase):
                     },
                     'errors': [{'shape': 'NoSuchResourceException'}],
                     'documentation': 'Docs for OperationTwo',
+                },
+                'PayloadOperation': {
+                    'http': {
+                        'method': 'POST',
+                        'requestUri': '/',
+                    },
+                    'name': 'PayloadOperation',
+                    'input': {
+                        'shape': 'PayloadOperationRequest'
+                    },
+                    'output': {
+                        'shape': 'PayloadOperationResponse',
+                    },
+                    'errors': [{'shape': 'NoSuchResourceException'}],
+                    'documentation': 'Docs for PayloadOperation',
+                },
+                'NoBodyOperation': {
+                    'http': {
+                        'method': 'GET',
+                        'requestUri': '/',
+                    },
+                    'name': 'NoBodyOperation',
+                    'input': {
+                        'shape': 'NoBodyOperationRequest'
+                    },
+                    'output': {
+                        'shape': 'OperationNameResponse',
+                    },
+                    'errors': [{'shape': 'NoSuchResourceException'}],
+                    'documentation': 'Docs for NoBodyOperation',
                 }
             },
             'shapes': {
@@ -167,13 +211,48 @@ class TestOperationModelFromService(unittest.TestCase):
                         }
                     }
                 },
+                'PayloadOperationRequest': {
+                    'type': 'structure',
+                    'members': {
+                        'Arg1': {'shape': 'TestConfig'},
+                        'Arg2': {'shape': 'stringType'},
+                    },
+                    'payload': 'Arg1'
+                },
+                'PayloadOperationResponse': {
+                    'type': 'structure',
+                    'members': {
+                        'String': {
+                            'shape': 'stringType',
+                        }
+                    },
+                    'payload': 'String'
+                },
+                'NoBodyOperationRequest': {
+                    'type': 'structure',
+                    'members': {
+                        'data': {
+                            'location': 'header',
+                            'locationName': 'x-amz-data',
+                            'shape': 'stringType'
+                        }
+                    }
+                },
                 'NoSuchResourceException': {
                     'type': 'structure',
                     'members': {}
                 },
                 'stringType': {
                     'type': 'string',
-                }
+                },
+                'TestConfig': {
+                    'type': 'structure',
+                    'members': {
+                        'timeout': {
+                            'shape': 'stringType'
+                        }
+                    }
+                },
             }
         }
         self.service_model = model.ServiceModel(self.model)
@@ -299,6 +378,23 @@ class TestOperationModelFromService(unittest.TestCase):
         operation_two = self.service_model.operation_model('OperationName')
         self.assertFalse(operation_two.is_endpoint_discovery_operation)
 
+    def test_endpoint_discovery_required(self):
+        operation = self.model['operations']['OperationName']
+        operation['endpointdiscovery'] = {'required': True}
+        service_model = model.ServiceModel(self.model)
+        self.assertTrue(service_model.endpoint_discovery_required)
+
+    def test_endpoint_discovery_required_false(self):
+        self.model['operations']['OperationName']['endpointdiscovery'] = {}
+        service_model = model.ServiceModel(self.model)
+        self.assertFalse(service_model.endpoint_discovery_required)
+
+    def test_endpoint_discovery_required_no_value(self):
+        operation = self.model['operations']['OperationName']
+        self.assertTrue(operation.get('endpointdiscovery') is None)
+        service_model = model.ServiceModel(self.model)
+        self.assertFalse(service_model.endpoint_discovery_required)
+
     def test_endpoint_discovery_present(self):
         operation = self.model['operations']['OperationName']
         operation['endpointdiscovery'] = {'required': True}
@@ -541,11 +637,15 @@ class TestDeepMerge(unittest.TestCase):
         # map_merged has a serialization as a member trait as well as
         # in the StrToStrMap.
         # The member trait should have precedence.
-        self.assertEqual(map_merged.serialization,
-                         # member beats the definition.
-                         {'name': 'Attribute',
-                          # From the definition.
-                          'flattened': True,})
+        self.assertEqual(
+            map_merged.serialization,
+            # member beats the definition.
+            {
+                'name': 'Attribute',
+                # From the definition.
+                'flattened': True,
+            }
+        )
         # Ensure we don't merge/mutate the original dicts.
         self.assertEqual(map_merged.key.serialization['name'], 'Name')
         self.assertEqual(map_merged.value.serialization['name'], 'Value')
@@ -664,7 +764,7 @@ class TestShapeResolver(unittest.TestCase):
                 }
             },
             'passwordType': {
-                "type":"string",
+                "type": "string",
             }
         }
         resolver = model.ShapeResolver(shapes)
@@ -675,6 +775,27 @@ class TestShapeResolver(unittest.TestCase):
                          ['NewPassword', 'OldPassword'])
         self.assertEqual(shape.members['OldPassword'].name, 'passwordType')
         self.assertEqual(shape.members['OldPassword'].type_name, 'string')
+        self.assertEqual(shape.error_code, None)
+
+    def test_exception_error_code(self):
+        shapes = {
+            'FooException': {
+                'exception': True,
+                'type': 'structure',
+                'members': {}
+            }
+        }
+        # Test without explicit error code
+        resolver = model.ShapeResolver(shapes)
+        shape = resolver.get_shape_by_name('FooException')
+        self.assertTrue(shape.metadata['exception'])
+        self.assertEqual(shape.error_code, 'FooException')
+        # Test with explicit error code
+        shapes['FooException']['error'] = {'code': 'ExceptionCode'}
+        resolver = model.ShapeResolver(shapes)
+        shape = resolver.get_shape_by_name('FooException')
+        self.assertTrue(shape.metadata['exception'])
+        self.assertEqual(shape.error_code, 'ExceptionCode')
 
     def test_shape_metadata(self):
         shapes = {
@@ -687,10 +808,10 @@ class TestShapeResolver(unittest.TestCase):
                 }
             },
             'passwordType': {
-                "type":"string",
-                "min":1,
-                "max":128,
-                "sensitive":True
+                "type": "string",
+                "min": 1,
+                "max": 128,
+                "sensitive": True
             }
         }
         resolver = model.ShapeResolver(shapes)
@@ -702,10 +823,30 @@ class TestShapeResolver(unittest.TestCase):
         self.assertEqual(member.metadata['max'], 128)
         self.assertEqual(member.metadata['sensitive'], True)
 
+    def test_error_shape_metadata(self):
+        shapes = {
+            'ResourceNotFoundException': {
+                'type': 'structure',
+                'members': {
+                    'message': {
+                        'shape': 'ErrorMessage',
+                    }
+                },
+                'exception': True,
+                'retryable': {'throttling': True}
+            }
+        }
+        resolver = model.ShapeResolver(shapes)
+        shape = resolver.get_shape_by_name('ResourceNotFoundException')
+        self.assertEqual(
+            shape.metadata,
+            {'exception': True, 'retryable': {'throttling': True}}
+        )
+
     def test_shape_list(self):
         shapes = {
             'mfaDeviceListType': {
-                "type":"list",
+                "type": "list",
                 "member": {"shape": "MFADevice"},
             },
             'MFADevice': {
