@@ -13,13 +13,14 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import botocore.config
-from tests import unittest, create_session, temporary_file
+from tests import mock, unittest, create_session, temporary_file, requires_crt
 import os
 import logging
 import tempfile
 import shutil
 
-import mock
+import pytest
+
 
 import botocore.session
 import botocore.exceptions
@@ -307,7 +308,7 @@ class TestBuiltinEventHandlers(BaseSessionTest):
         self.foo_called = True
 
     def tearDown(self):
-        super(TestBuiltinEventHandlers, self).setUp()
+        super(TestBuiltinEventHandlers, self).tearDown()
         self.handler_patch.stop()
 
     def test_registered_builtin_handlers(self):
@@ -379,6 +380,22 @@ class TestSessionPartitionFiles(BaseSessionTest):
         regions = self.session.get_available_regions('__foo__')
         self.assertEqual([], regions)
 
+    def test_provides_correct_partition_for_region(self):
+        partition = self.session.get_partition_for_region('us-west-2')
+        self.assertEqual(partition, 'aws')
+
+    def test_provides_correct_partition_for_region_regex(self):
+        partition = self.session.get_partition_for_region('af-south-99')
+        self.assertEqual(partition, 'aws')
+
+    def test_provides_correct_partition_for_region_non_default(self):
+        partition = self.session.get_partition_for_region('cn-north-1')
+        self.assertEqual(partition, 'aws-cn')
+
+    def test_raises_exception_for_invalid_region(self):
+        with self.assertRaises(botocore.exceptions.UnknownRegionError):
+            self.session.get_partition_for_region('no-good-1')
+
 
 class TestSessionUserAgent(BaseSessionTest):
     def test_can_change_user_agent_name(self):
@@ -408,6 +425,22 @@ class TestSessionUserAgent(BaseSessionTest):
         self.assertTrue(user_agent.endswith('custom-thing/other'))
         self.assertIn('exec-env/FooEnv', user_agent)
 
+    @requires_crt()
+    def test_crt_user_agent_appended(self):
+        user_agent = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertNotIn('awscrt/Unknown', user_agent)
+
+    @requires_crt()
+    def test_crt_and_extra_user_agent(self):
+        user_agent = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertNotIn('custom-thing/other', user_agent)
+        self.session.user_agent_extra = 'custom-thing/other'
+        user_agent_w_extra = self.session.user_agent()
+        self.assertIn(' awscrt/', user_agent)
+        self.assertTrue(user_agent_w_extra.endswith('custom-thing/other'))
+
 
 class TestConfigLoaderObject(BaseSessionTest):
     def test_config_loader_delegation(self):
@@ -659,10 +692,10 @@ class TestCreateClient(BaseSessionTest):
             del self.environ['FOO_PROFILE']
             self.environ['FOO_CONFIG_FILE'] = f.name
             f.write('[default]\n')
-            f.write('foo_api_versions =\n'
-                    '    myservice = %s\n'
-                    '    myservice2 = %s\n' % (
-                        config_api_version, second_config_api_version)
+            f.write(
+                f'foo_api_versions =\n'
+                f'    myservice = {config_api_version}\n'
+                f'    myservice2 = {second_config_api_version}\n'
             )
             f.flush()
 
@@ -703,21 +736,29 @@ class TestSessionComponent(BaseSessionTest):
         self.assertIs(
             self.session._get_internal_component('internal'), component)
         with self.assertRaises(ValueError):
-            self.session.get_component('internal')
+            # get_component has been deprecated to the public
+            with pytest.warns(DeprecationWarning):
+                self.session.get_component('internal')
 
     def test_internal_endpoint_resolver_is_same_as_deprecated_public(self):
         endpoint_resolver = self.session._get_internal_component(
             'endpoint_resolver')
-        self.assertIs(
-            self.session.get_component('endpoint_resolver'), endpoint_resolver)
+        # get_component has been deprecated to the public
+        with pytest.warns(DeprecationWarning):
+            self.assertIs(
+                self.session.get_component('endpoint_resolver'),
+                endpoint_resolver
+            )
 
     def test_internal_exceptions_factory_is_same_as_deprecated_public(self):
         exceptions_factory = self.session._get_internal_component(
             'exceptions_factory')
-        self.assertIs(
-            self.session.get_component('exceptions_factory'),
-            exceptions_factory
-        )
+        # get_component has been deprecated to the public
+        with pytest.warns(DeprecationWarning):
+            self.assertIs(
+                self.session.get_component('exceptions_factory'),
+                exceptions_factory
+            )
 
 
 class TestClientMonitoring(BaseSessionTest):
@@ -729,9 +770,10 @@ class TestClientMonitoring(BaseSessionTest):
             client.meta.events)
 
     def assert_monitoring_host_and_port(self, session, host, port):
-        with mock.patch('botocore.monitoring.SocketPublisher',
-                        spec=True) as mock_publisher:
-            client = session.create_client('ec2', 'us-west-2')
+        with mock.patch(
+            'botocore.monitoring.SocketPublisher', spec=True
+        ) as mock_publisher:
+            session.create_client('ec2', 'us-west-2')
         self.assertEqual(mock_publisher.call_count, 1)
         _, args, kwargs = mock_publisher.mock_calls[0]
         self.assertEqual(kwargs.get('host'), host)
@@ -863,16 +905,16 @@ class TestSessionRegionSetup(BaseSessionTest):
     def test_new_session_with_valid_region(self):
         s3_client = self.session.create_client('s3', 'us-west-2')
         self.assertIsInstance(s3_client, client.BaseClient)
-        self.assertEquals(s3_client.meta.region_name, 'us-west-2')
+        self.assertEqual(s3_client.meta.region_name, 'us-west-2')
 
     def test_new_session_with_unknown_region(self):
         s3_client = self.session.create_client('s3', 'MyCustomRegion1')
         self.assertIsInstance(s3_client, client.BaseClient)
-        self.assertEquals(s3_client.meta.region_name, 'MyCustomRegion1')
+        self.assertEqual(s3_client.meta.region_name, 'MyCustomRegion1')
 
     def test_new_session_with_invalid_region(self):
         with self.assertRaises(botocore.exceptions.InvalidRegionError):
-            s3_client = self.session.create_client('s3', 'not.a.real#region')
+            self.session.create_client('s3', 'not.a.real#region')
 
     def test_new_session_with_none_region(self):
         s3_client = self.session.create_client('s3', region_name=None)
